@@ -24,7 +24,6 @@ val dump_memo_graph_with_timing : t -> bool
 val watch : t -> Dune_rpc_impl.Watch_mode_config.t
 val file_watcher : t -> Dune_engine.Scheduler.Run.file_watcher
 val prefix_target : t -> string -> string
-val insignificant_changes : t -> [ `React | `Ignore ]
 
 module Action_runner : sig
   type t =
@@ -49,8 +48,6 @@ module Builder : sig
   val term : t Cmdliner.Term.t
 end
 
-val build : Builder.t -> t
-
 (** [init] creates a [Common.t] by executing a sequence of side-effecting actions to
     initialize Dune's working environment based on the options determined in the\
     [Builder.t].
@@ -87,3 +84,7 @@ module Let_syntax : sig
   val ( let+ ) : 'a Cmdliner.Term.t -> ('a -> 'b) -> 'b Cmdliner.Term.t
   val ( and+ ) : 'a Cmdliner.Term.t -> 'b Cmdliner.Term.t -> ('a * 'b) Cmdliner.Term.t
 end
+
+(** [one_of term1 term2] allows options from [term1] or exclusively options from
+    [term2]. If the user passes options from both terms, an error is reported. *)
+val one_of : 'a Cmdliner.Term.t -> 'a Cmdliner.Term.t -> 'a Cmdliner.Term.t
