@@ -102,12 +102,12 @@ module Status_line = struct
   let () = at_exit (fun () -> Printf.printf "\r%*s\r" (String.length !displayed) "")
 end
 
-(* Return list of entries in [path] as [path/entry] *)
+(* Return a sorted list of entries in [path] as [path/entry] *)
 let readdir path =
-  Array.fold_right
-    ~f:(fun entry dir -> (path ^/ entry) :: dir)
-    ~init:[]
-    (Sys.readdir path)
+  Sys.readdir path
+  |> Array.to_list
+  |> List.map ~f:(fun entry -> path ^/ entry)
+  |> List.sort ~cmp:String.compare
 ;;
 
 let open_out file =
@@ -445,7 +445,9 @@ end = struct
 
     let open_temp_file () =
       let out = Temp.file "duneboot-" ".output" in
-      let fd = Unix.openfile out [ O_WRONLY; O_CREAT; O_TRUNC; O_SHARE_DELETE ] 0o666 in
+      let fd =
+        Unix.openfile out [ O_WRONLY; O_CREAT; O_TRUNC; O_SHARE_DELETE; O_CLOEXEC ] 0o666
+      in
       out, fd
     ;;
 
@@ -1065,6 +1067,7 @@ let build
   ~ocaml_config
   ~dependencies
   ~c_files
+  ~build_flags
   ~link_flags
   { target = name, main; external_libraries; _ }
   =
@@ -1112,7 +1115,7 @@ let build
         Process.run
           ~cwd:build_dir
           Config.compiler
-          (List.concat [ [ "-c"; "-g" ]; external_includes; [ file ] ])
+          (List.concat [ [ "-c"; "-g" ]; external_includes; build_flags; [ file ] ])
         >>| fun () -> Filename.chop_extension file ^ ext_obj))
   >>= fun obj_files ->
   let compiled_ml_ext =
@@ -1139,27 +1142,6 @@ let build
        ])
 ;;
 
-let build_with_single_command
-  ~ocaml_config:_
-  ~dependencies
-  ~c_files
-  ~link_flags
-  { target = name, main; external_libraries; _ }
-  =
-  let external_libraries, external_includes = resolve_externals external_libraries in
-  write_args "mods_list" (sort_files dependencies ~main);
-  Process.run
-    ~cwd:build_dir
-    Config.compiler
-    (List.concat
-       [ common_build_args name ~external_includes ~external_libraries
-       ; [ "-no-alias-deps"; "-w"; "-49-6" ]
-       ; c_files
-       ; [ "-args"; "mods_list" ] @ link_flags
-       ; allow_unstable_sources
-       ])
-;;
-
 let rec rm_rf fn =
   match Unix.lstat fn with
   | { st_kind = S_DIR; _ } ->
@@ -1170,6 +1152,12 @@ let rec rm_rf fn =
 
 and clear dir = List.iter (readdir dir) ~f:rm_rf
 
+let get_flags ocaml_system flags =
+  match List.assoc_opt ocaml_system flags with
+  | None -> []
+  | Some flags -> flags
+;;
+
 (** {2 Bootstrap process} *)
 let main () =
   (try clear build_dir with
@@ -1183,16 +1171,14 @@ let main () =
   let c_files = List.map ~f:(fun (_, _, c_files) -> c_files) libraries |> List.concat in
   get_dependencies libraries
   >>= fun dependencies ->
-  let link_flags =
+  let ocaml_system =
     match StringMap.find_opt "system" ocaml_config with
     | None -> assert false
-    | Some platform ->
-      (match List.assoc_opt platform Libs.link_flags with
-       | None -> []
-       | Some flags -> flags)
+    | Some s -> s
   in
-  let build = if concurrency = 1 || Sys.win32 then build_with_single_command else build in
-  build ~ocaml_config ~dependencies ~c_files ~link_flags task
+  let build_flags = get_flags ocaml_system Libs.build_flags in
+  let link_flags = get_flags ocaml_system Libs.link_flags in
+  build ~ocaml_config ~dependencies ~c_files ~build_flags ~link_flags task
 ;;
 
 let () = Fiber.run (main ())
