@@ -117,7 +117,7 @@ module Conf = struct
 
   let of_context (context : Context.t) =
     let open Memo.O in
-    let get_vcs = Source_tree.nearest_vcs in
+    let get_vcs = Vcs_db.nearest_vcs in
     let name = Context.name context in
     let get_location = Install.Paths.get_local_location name in
     let get_config_path = function
@@ -140,7 +140,7 @@ module Conf = struct
 
   let of_install ~relocatable ~roots ~(context : Context.t) =
     let open Memo.O in
-    let get_vcs = Source_tree.nearest_vcs in
+    let get_vcs = Vcs_db.nearest_vcs in
     let hardcoded_ocaml_path =
       match relocatable with
       | Some prefix -> Memo.return @@ Relocatable prefix
@@ -149,7 +149,7 @@ module Conf = struct
         Hardcoded default_ocamlpath
     in
     let get_location section package =
-      let paths = Install.Paths.make ~package ~roots in
+      let paths = Install.Paths.make ~relative:Path.relative ~package ~roots in
       Install.Paths.get paths section
     in
     let get_config_path = function
@@ -175,10 +175,18 @@ module Conf = struct
     match has_subst with
     | No_substitution -> Fiber.return ()
     | Some_substitution ->
-      Memo.run t.sign_hook
-      |> Fiber.bind ~f:(function
-        | Some hook -> hook file
-        | None -> Fiber.return ())
+      let executable =
+        match Path.Untracked.stat file with
+        | Error _ -> false
+        | Ok { st_perm; _ } -> Path.Permissions.test Path.Permissions.execute st_perm
+      in
+      if executable
+      then
+        Memo.run t.sign_hook
+        |> Fiber.bind ~f:(function
+          | Some hook -> hook file
+          | None -> Fiber.return ())
+      else Fiber.return ()
   ;;
 end
 
@@ -638,6 +646,8 @@ let copy ~conf ~input_file ~input ~output =
 ;;
 
 let copy_file_non_atomic ~conf ?chmod ~src ~dst () =
+  (* CR-rgrinberg: our copying here is slow. If we scan the file and detect no
+     substitutions, we should go directly to [Io.copy_file] *)
   let open Fiber.O in
   let* ic, oc = Fiber.return (Io.setup_copy ?chmod ~src ~dst ()) in
   Fiber.finalize
@@ -653,7 +663,7 @@ let copy_file_non_atomic ~conf ?chmod ~src ~dst () =
 let replace_if_different ~delete_dst_if_it_is_a_directory ~src ~dst =
   let up_to_date =
     match Path.Untracked.stat dst with
-    | Ok { st_kind; _ } when st_kind = S_DIR ->
+    | Ok { st_kind = S_DIR; _ } ->
       (match delete_dst_if_it_is_a_directory with
        | true ->
          Path.rm_rf dst;
@@ -673,15 +683,7 @@ let replace_if_different ~delete_dst_if_it_is_a_directory ~src ~dst =
   if not up_to_date then Path.rename src dst
 ;;
 
-let copy_file
-  ~conf
-  ?(executable = false)
-  ?chmod
-  ?(delete_dst_if_it_is_a_directory = false)
-  ~src
-  ~dst
-  ()
-  =
+let copy_file ~conf ?chmod ?(delete_dst_if_it_is_a_directory = false) ~src ~dst () =
   (* We create a temporary file in the same directory to ensure it's on the same
      partition as [dst] (otherwise, [Path.rename temp_file dst] won't work). The
      prefix ".#" is used because Dune ignores such files and so creating this
@@ -696,11 +698,7 @@ let copy_file
       let open Fiber.O in
       Path.parent dst |> Option.iter ~f:Path.mkdir_p;
       let* has_subst = copy_file_non_atomic ~conf ?chmod ~src ~dst:temp_file () in
-      let+ () =
-        if executable
-        then Conf.run_sign_hook conf ~has_subst temp_file
-        else Fiber.return ()
-      in
+      let+ () = Conf.run_sign_hook conf ~has_subst temp_file in
       replace_if_different ~delete_dst_if_it_is_a_directory ~src:temp_file ~dst)
     ~finally:(fun () ->
       Path.unlink_no_err temp_file;
