@@ -95,6 +95,7 @@ let coqc ~loc ~dir ~sctx =
   Super_context.resolve_program_memo
     sctx
     "coqc"
+    ~where:Original_path
     ~dir
     ~loc:(Some loc)
     ~hint:"opam install coq"
@@ -125,13 +126,55 @@ let select_native_mode ~sctx ~dir (buildable : Coq_stanza.Buildable.t) =
           | _ -> Coq_mode.VoOnly))
 ;;
 
-let coq_flags ~dir ~stanza_flags ~expander ~sctx =
-  let open Action_builder.O in
-  let* standard =
-    Action_builder.of_memo
-    @@ (Super_context.env_node ~dir sctx |> Memo.bind ~f:Env_node.coq)
+let coq_env =
+  let f =
+    Env_stanza_db_flags.flags
+      ~name:"coq-env"
+      ~root:(fun _ _ -> Memo.return (Action_builder.return Coq_flags.default))
+      ~f:(fun ~parent expander (config : Dune_env.config) ->
+        Memo.return
+        @@
+        let open Action_builder.O in
+        let+ coq_flags =
+          let standard =
+            let+ x = Action_builder.of_memo_join parent in
+            x.coq_flags
+          in
+          Expander.expand_and_eval_set expander (Coq_env.flags config.coq) ~standard
+        and+ coqdoc_flags =
+          let standard =
+            let+ x = Action_builder.of_memo_join parent in
+            x.coqdoc_flags
+          in
+          Expander.expand_and_eval_set
+            expander
+            (Coq_env.coqdoc_flags config.coq)
+            ~standard
+        in
+        { Coq_flags.coq_flags; coqdoc_flags })
   in
-  Expander.expand_and_eval_set expander stanza_flags ~standard
+  fun ~dir ->
+    (let* () = Memo.return () in
+     (Staged.unstage f) dir)
+    |> Action_builder.of_memo_join
+;;
+
+let coq_flags ~dir ~stanza_flags ~expander =
+  Expander.expand_and_eval_set
+    expander
+    stanza_flags
+    ~standard:
+      (Action_builder.map ~f:(fun { Coq_flags.coq_flags; _ } -> coq_flags) (coq_env ~dir))
+;;
+
+let coqdoc_flags ~dir ~stanza_coqdoc_flags ~expander =
+  Expander.expand_and_eval_set
+    expander
+    stanza_coqdoc_flags
+    ~standard:
+      (Action_builder.map
+         ~f:(fun { Coq_flags.coqdoc_flags; _ } -> coqdoc_flags)
+         (coq_env ~dir))
 ;;
 
 let theory_coqc_flag lib =
@@ -148,23 +191,98 @@ let theories_flags ~theories_deps =
      Command.Args.S (List.map ~f:theory_coqc_flag libs))
 ;;
 
-let boot_flags ~coq_lang_version t : _ Command.Args.t =
-  let boot_flag = if coq_lang_version >= (0, 8) then [ "-boot" ] else [] in
-  match t with
-  (* We are compiling the prelude itself
-     [should be replaced with (per_file ...) flags] *)
-  | `Bootstrap_prelude -> As ("-noinit" :: boot_flag)
-  (* No special case *)
-  | `No_boot | `Bootstrap _ -> As boot_flag
-;;
+module Bootstrap : sig
+  type t =
+    | Implicit (** Use implicit stdlib loading, for coq lang versions < 0.8 *)
+    | No_stdlib
+    (** We are in >= 0.8, however the user set stdlib = no
+        , or we are compiling the prelude *)
+    | Stdlib of Coq_lib.t (** Regular case in >= 0.8 (or in < 0.8
+                              (boot) was used *)
 
-let coqc_file_flags
-  ~dir
-  ~theories_deps
-  ~wrapper_name
-  ~boot_type
-  ~ml_flags
-  ~coq_lang_version
+  val empty : t
+
+  val make
+    :  scope:Scope.t
+    -> use_stdlib:bool
+    -> wrapper_name:string
+    -> coq_lang_version:Syntax.Version.t
+    -> Coq_module.t
+    -> t Resolve.Memo.t
+
+  val flags : t -> 'a Command.Args.t
+end = struct
+  type t =
+    | Implicit (** Use implicit stdlib loading, for coq lang versions < 0.8 *)
+    | No_stdlib
+    (** We are in >= 0.8, however the user set stdlib = no
+        , or we are compiling the prelude *)
+    | Stdlib of Coq_lib.t (** Regular case in >= 0.8 (or in < 0.8
+                              (boot) was used *)
+
+  (* For empty set of modules, we return Prelude which is kinda
+     conservative. *)
+  let empty = No_stdlib
+
+  (* Hack to know if a module is a prelude module *)
+  let check_init_module bootlib wrapper_name coq_module =
+    String.equal (Coq_lib_name.wrapper (Coq_lib.name bootlib)) wrapper_name
+    && Option.equal
+         String.equal
+         (List.hd_opt (Coq_module.prefix coq_module))
+         (Some "Init")
+  ;;
+
+  (* [Bootstrap.t] determines, for a concrete Coq module, how the Coq
+     "standard library" is being handled. See the main modes above. *)
+  let make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module =
+    let open Resolve.Memo.O in
+    let* boot_lib =
+      Scope.coq_libs scope
+      |> Resolve.Memo.lift_memo
+      >>= Coq_lib.DB.resolve_boot ~coq_lang_version
+    in
+    if use_stdlib
+    then (
+      match boot_lib with
+      | None ->
+        if coq_lang_version >= (0, 8)
+        then
+          Resolve.Memo.fail
+            (User_message.make
+               [ Pp.text
+                   "Couldn't find Coq standard library, and theory is not using (stdlib \
+                    no)"
+               ])
+        else Resolve.Memo.return Implicit
+      | Some (_loc, boot_lib) ->
+        Resolve.Memo.return
+        @@
+        (* TODO: replace with per_file flags *)
+        let init = check_init_module boot_lib wrapper_name coq_module in
+        if init
+        then No_stdlib
+        else if coq_lang_version >= (0, 8)
+        then Stdlib boot_lib
+        else (
+          (* Check if the boot library is in scope or not; note!!
+             This will change once we install dune-package files
+             for installed libs *)
+          match boot_lib with
+          | Legacy _ -> Implicit
+          | Dune _ -> Stdlib boot_lib))
+    else Resolve.Memo.return No_stdlib
+  ;;
+
+  let flags t : _ Command.Args.t =
+    match t with
+    | Implicit -> As []
+    | Stdlib _ -> As [ "-boot" ]
+    | No_stdlib -> As [ "-noinit"; "-boot" ]
+  ;;
+end
+
+let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
   : _ Command.Args.t list
   =
   let file_flags : _ Command.Args.t list =
@@ -175,7 +293,8 @@ let coqc_file_flags
     ; A wrapper_name
     ]
   in
-  [ boot_flags ~coq_lang_version boot_type; S file_flags ]
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  [ Dyn boot_flags; S file_flags ]
 ;;
 
 let native_includes ~dir =
@@ -204,7 +323,7 @@ let directories_of_lib ~sctx lib =
        EJGA: We need to understand how this interacts with globally
        installed stuff, that's more tricky than it looks actually!
 
-       This fuction is used in order to determine the -nI flags that
+       This function is used in order to determine the -nI flags that
        Coq native compiler will pass to OCaml so it can find the .cmxs
        files. For things in user-contrib we don't need to pass these
        flags but only because Coq has a very large hack adding this
@@ -334,44 +453,13 @@ let ml_flags_and_ml_pack_rule
   Resolve.Memo.map ~f:fst res, mlpack_rule
 ;;
 
-(* the internal boot flag determines if the Coq "standard library" is being
-   built, in case we need to explicitly tell Coq where the build artifacts are
-   and add `Init.Prelude.vo` as a dependency; there is a further special case
-   when compiling the prelude, in this case we also need to tell Coq not to
-   try to load the prelude. *)
-let boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module =
-  let* scope = Scope.DB.find_by_dir dir in
-  let+ boot_lib =
-    Scope.coq_libs scope
-    >>= Coq_lib.DB.resolve_boot ~coq_lang_version
-    |> Resolve.Memo.read_memo
-  in
-  if use_stdlib
-  then (
-    match boot_lib with
-    | None ->
-      `No_boot
-        (* XXX: use_stdlib + no_boot is
-           actually a workspace error, cleanup *)
-    | Some (_loc, lib) ->
-      (* This is here as an optimization, TODO; replace with per_file flags *)
-      let init =
-        String.equal (Coq_lib_name.wrapper (Coq_lib.name lib)) wrapper_name
-        && Option.equal
-             String.equal
-             (List.hd_opt (Coq_module.prefix coq_module))
-             (Some "Init")
-      in
-      if init then `Bootstrap_prelude else `Bootstrap lib)
-  else `Bootstrap_prelude
-;;
-
 let dep_theory_file ~dir ~wrapper_name =
   Path.Build.relative dir ("." ^ wrapper_name)
   |> Path.Build.set_extension ~ext:".theory.d"
 ;;
 
 let setup_coqdep_for_theory_rule
+  ~scope
   ~sctx
   ~dir
   ~loc
@@ -384,25 +472,19 @@ let setup_coqdep_for_theory_rule
   ~coq_lang_version
   coq_modules
   =
-  let* boot_type =
+  let boot_type =
     (* If coq_modules are empty it doesn't really matter, so we take
        the more conservative path and pass -boot, we don't care here
        about -noinit as coqdep ignores it *)
     match coq_modules with
-    | [] -> Memo.return `Bootstrap_prelude
-    | m :: _ -> boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version m
+    | [] -> Resolve.Memo.return Bootstrap.empty
+    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
   in
   (* coqdep needs the full source + plugin's mlpack to be present :( *)
   let sources = List.rev_map ~f:Coq_module.source coq_modules in
   let file_flags =
     [ Command.Args.S
-        (coqc_file_flags
-           ~dir
-           ~theories_deps
-           ~wrapper_name
-           ~boot_type
-           ~ml_flags
-           ~coq_lang_version)
+        (coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags)
     ; As [ "-dyndep"; "opt"; "-vos" ]
     ; Deps sources
     ]
@@ -413,6 +495,7 @@ let setup_coqdep_for_theory_rule
       sctx
       "coqdep"
       ~dir
+      ~where:Original_path
       ~loc:(Some loc)
       ~hint:"opam install coq"
   in
@@ -435,6 +518,16 @@ let coqdep_invalid phase line =
     [ "phase", Dyn.string phase; "line", Dyn.string line ]
 ;;
 
+(* Handle the case where the path contains ":" and coqdep escapes this
+   as "\:" causing Dune to misinterpret the path. We revert
+   the escaping, which allows dune to work on Windows.
+
+   Note that coqdep escapes a few more things, including spaces, $, #,
+   [], ?, %, homedir... How to handle that seems tricky.
+*)
+let escaped_colon = Re.compile (Re.str "\\:")
+let unescape_coqdep string = Re.replace_string escaped_colon ~by:":" string
+
 let parse_line ~dir line =
   match String.lsplit2 line ~on:':' with
   | None -> coqdep_invalid "split" line
@@ -448,7 +541,9 @@ let parse_line ~dir line =
     in
     (* let depname, ext = Filename.split_extension ff in *)
     let target = Path.relative (Path.build dir) target in
-    let deps = String.extract_blank_separated_words deps in
+    (* EJGA: XXX using `String.extract_blank_separated_words` works
+       for OCaml, but not for Coq as we don't use `-modules` *)
+    let deps = unescape_coqdep deps |> String.extract_blank_separated_words in
     (* Add prelude deps for when stdlib is in scope and we are not actually
        compiling the prelude *)
     let deps = List.map ~f:(Path.relative (Path.build dir)) deps in
@@ -475,6 +570,26 @@ let get_dep_map ~dir ~wrapper_name : Path.t list Dep_map.t Action_builder.t =
       ]
 ;;
 
+(* EJGA: Would be nice to have a functor in [Stdune.Tuple] to do this *)
+module DWInput : Memo.Input with type t = Path.Build.t * string = struct
+  type t = Path.Build.t * string
+
+  let hash = Tuple.T2.hash Path.Build.hash String.hash
+  let equal = Tuple.T2.equal Path.Build.equal String.equal
+  let to_dyn = Tuple.T2.to_dyn Path.Build.to_dyn String.to_dyn
+end
+
+(* EJGA: with this code, we have a combined memo node that will be
+   re-executed iff [dir], [wrapper_name], or the deps of the original
+   action builder (the [.v.d] file from [dep_theory_file ~dir
+   ~wrapper_name]) change *)
+let memo_get_dep_map =
+  Action_builder.create_memo
+    "coq_dep_map"
+    ~input:(module DWInput)
+    (fun (dir, wrapper_name) -> get_dep_map ~dir ~wrapper_name)
+;;
+
 let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
   let open Action_builder.O in
   let vo_target =
@@ -485,7 +600,8 @@ let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
     in
     Path.set_extension ~ext (Coq_module.source coq_module)
   in
-  let* dep_map = get_dep_map ~dir ~wrapper_name in
+  let* dep_map = Action_builder.exec_memo memo_get_dep_map (dir, wrapper_name) in
+  let* boot_type = Resolve.Memo.read boot_type in
   match Dep_map.find dep_map vo_target with
   | None ->
     Code_error.raise
@@ -498,8 +614,8 @@ let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
     let deps =
       let prelude = "Init/Prelude.vo" in
       match boot_type with
-      | `No_boot | `Bootstrap_prelude -> deps
-      | `Bootstrap lib -> Path.relative (Coq_lib.obj_root lib) prelude :: deps
+      | Bootstrap.No_stdlib | Bootstrap.Implicit -> deps
+      | Bootstrap.Stdlib lib -> Path.relative (Coq_lib.obj_root lib) prelude :: deps
     in
     Action_builder.paths deps
 ;;
@@ -515,13 +631,12 @@ let generic_coq_args
   ~ml_flags
   ~theories_deps
   ~theory_dirs
-  ~coq_lang_version
   coq_module
   =
   let+ coq_stanza_flags =
     let+ expander = Super_context.expander sctx ~dir in
     let coq_flags =
-      let coq_flags = coq_flags ~expander ~dir ~stanza_flags ~sctx in
+      let coq_flags = coq_flags ~expander ~dir ~stanza_flags in
       (* By default we have the -q flag. We don't want to pass this to coqtop to
          allow users to load their .coqrc files for interactive development.
          Therefore we manually scrub the -q setting when passing arguments to
@@ -549,13 +664,7 @@ let generic_coq_args
     coqc_native_flags ~sctx ~dir ~theories_deps ~theory_dirs ~mode
   in
   let file_flags =
-    coqc_file_flags
-      ~dir
-      ~theories_deps
-      ~wrapper_name
-      ~boot_type
-      ~ml_flags
-      ~coq_lang_version
+    coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
   in
   match coq_prog with
   | `Coqc ->
@@ -568,6 +677,7 @@ let generic_coq_args
 ;;
 
 let setup_coqc_rule
+  ~scope
   ~loc
   ~dir
   ~sctx
@@ -584,8 +694,8 @@ let setup_coqc_rule
   coq_module
   =
   (* Process coqdep and generate rules *)
-  let* boot_type =
-    boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   let* coqc = coqc ~loc ~dir ~sctx in
   let obj_files =
@@ -609,7 +719,6 @@ let setup_coqc_rule
       ~theories_deps
       ~theory_dirs
       ~mode
-      ~coq_lang_version
       ~coq_prog:`Coqc
       coq_module
   in
@@ -668,6 +777,7 @@ let setup_coqdoc_rules ~sctx ~dir ~theories_deps (s : Coq_stanza.Theory.t) coq_m
             sctx
             "coqdoc"
             ~dir
+            ~where:Original_path
             ~loc:(Some loc)
             ~hint:"opam install coq"
         in
@@ -697,9 +807,11 @@ let setup_coqdoc_rules ~sctx ~dir ~theories_deps (s : Coq_stanza.Theory.t) coq_m
            in
            let extra_coqdoc_flags =
              (* Standard flags for coqdoc *)
-             let standard = Action_builder.return [ "--toc" ] in
              let open Action_builder.O in
              let* expander = Action_builder.of_memo @@ Super_context.expander sctx ~dir in
+             let standard =
+               coqdoc_flags ~dir ~stanza_coqdoc_flags:s.coqdoc_flags ~expander
+             in
              Expander.expand_and_eval_set expander s.coqdoc_flags ~standard
            in
            [ Command.Args.S file_flags
@@ -773,7 +885,7 @@ let extraction_context
       ~f:(Coq_lib.DB.resolve coq_lib_db ~coq_lang_version)
   in
   (* Extraction requires a boot library so we do this unconditionally
-     for now. We must do this because it can happne that
+     for now. We must do this because it can happen that
      s.buildable.theories is empty *)
   let boot =
     let* coq_lib_db = coq_lib_db in
@@ -827,6 +939,7 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   (* First we setup the rule calling coqdep *)
   setup_coqdep_for_theory_rule
+    ~scope
     ~sctx
     ~dir
     ~loc
@@ -842,6 +955,7 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
         coq_modules
         ~f:
           (setup_coqc_rule
+             ~scope
              ~loc
              ~dir
              ~sctx
@@ -871,8 +985,8 @@ let coqtop_args_theory ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) coq_mo
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   let name = snd s.name in
   let use_stdlib = s.buildable.use_stdlib in
-  let* boot_type =
-    boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   let* coq_dir_contents = Dir_contents.coq dir_contents in
   let theory_dirs =
@@ -889,7 +1003,6 @@ let coqtop_args_theory ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) coq_mo
     ~ml_flags
     ~theories_deps
     ~theory_dirs
-    ~coq_lang_version
     coq_module
 ;;
 
@@ -996,6 +1109,7 @@ let setup_coqpp_rules ~sctx ~dir ({ loc; modules } : Coq_stanza.Coqpp.t) =
     Super_context.resolve_program_memo
       sctx
       "coqpp"
+      ~where:Original_path
       ~dir
       ~loc:(Some loc)
       ~hint:"opam install coq"
@@ -1034,6 +1148,7 @@ let setup_extraction_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Extraction.t
   in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   setup_coqdep_for_theory_rule
+    ~scope
     ~sctx
     ~dir
     ~loc
@@ -1046,6 +1161,7 @@ let setup_extraction_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Extraction.t
     ~coq_lang_version
     [ coq_module ]
   >>> setup_coqc_rule
+        ~scope
         ~file_targets
         ~stanza_flags:s.buildable.flags
         ~sctx
@@ -1071,8 +1187,8 @@ let coqtop_args_extraction ~sctx ~dir (s : Coq_stanza.Extraction.t) coq_module =
     extraction_context ~context ~scope ~coq_lang_version s.buildable
   in
   let wrapper_name = "DuneExtraction" in
-  let* boot_type =
-    boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   generic_coq_args
@@ -1086,16 +1202,15 @@ let coqtop_args_extraction ~sctx ~dir (s : Coq_stanza.Extraction.t) coq_module =
     ~ml_flags
     ~theories_deps
     ~theory_dirs:Path.Build.Set.empty
-    ~coq_lang_version
     coq_module
 ;;
 
 (* Version for export *)
 let deps_of ~dir ~use_stdlib ~wrapper_name ~mode ~coq_lang_version coq_module =
-  let open Action_builder.O in
-  let* boot_type =
-    Action_builder.of_memo
-      (boot_type ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module)
+  let boot_type =
+    let open Memo.O in
+    let* scope = Scope.DB.find_by_dir dir in
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module
 ;;
