@@ -45,22 +45,12 @@ module Lib = struct
     ; external_location : External_location.t option
     }
 
-  let make ~info ~main_module_name ~external_location =
-    let obj_dir = Lib_info.obj_dir info in
-    let dir = Obj_dir.dir obj_dir in
-    let map_path p =
-      if Path.is_managed p then Path.relative dir (Path.basename p) else p
-    in
-    let info = Lib_info.map_path info ~f:map_path in
-    { info; main_module_name; external_location }
-  ;;
-
   let of_dune_lib ~info ~main_module_name =
-    make ~info ~main_module_name ~external_location:None
+    { info; main_module_name; external_location = None }
   ;;
 
   let of_findlib info external_location =
-    make ~info ~main_module_name:None ~external_location:(Some external_location)
+    { info; main_module_name = None; external_location = Some external_location }
   ;;
 
   let dir_of_name name =
@@ -223,7 +213,7 @@ module Lib = struct
        and+ requires = field_l "requires" (Lib_dep.decode ~allow_re_export:true)
        and+ ppx_runtime_deps = libs "ppx_runtime_deps"
        and+ virtual_ = field_b "virtual"
-       and+ sub_systems = Sub_system_info.record_parser ()
+       and+ sub_systems = Sub_system_info.record_parser
        and+ orig_src_dir = field_o "orig_src_dir" path
        and+ modules = field "modules" (Modules.decode ~src_dir:base)
        and+ special_builtin_support =
@@ -238,7 +228,7 @@ module Lib = struct
        let entry_modules = Modules.entry_modules modules |> List.map ~f:Module.name in
        let info : Path.t Lib_info.t =
          let src_dir = Obj_dir.dir obj_dir in
-         let enabled = Lib_info.Enabled_status.Normal in
+         let enabled = Memo.return Lib_info.Enabled_status.Normal in
          let status =
            match Lib_name.analyze name with
            | Private (_, _) -> Lib_info.Status.Installed_private
