@@ -1,3 +1,5 @@
+module Alias_builder = Alias_builder
+module Action_builder = Action_builder
 module Alias = Alias0
 module Findlib = Findlib
 module Main = Main
@@ -5,7 +7,12 @@ module Context = Context
 module Env_node = Env_node
 module Link_flags = Link_flags
 module Ocaml_flags = Ocaml_flags
+module Ocaml_flags_db = Ocaml_flags_db
 module Js_of_ocaml = Js_of_ocaml
+module Menhir_env = Menhir_env
+module Menhir_rules = Menhir_rules
+module Foreign_rules = Foreign_rules
+module Jsoo_rules = Jsoo_rules
 module Super_context = Super_context
 module Compilation_context = Compilation_context
 module Colors = Colors
@@ -46,21 +53,22 @@ module Dep_graph = Dep_graph
 module Lib_config = Lib_config
 module Preprocess = Preprocess
 module Preprocessing = Preprocessing
-module Coq_mode = Coq_mode
-module Coq_rules = Coq_rules
-module Coq_module = Coq_module
-module Coq_sources = Coq_sources
-module Coq_lib_name = Coq_lib_name
-module Coq_lib = Coq_lib
 module Command = Command
 module Clflags = Clflags
 module Dune_project = Dune_project
+module Dune_project_name = Dune_project_name
+module Vcs_db = Vcs_db
 module Source_tree = Source_tree
-module Sub_dirs = Sub_dirs
+module Source_dir_status = Source_dir_status
+module Dune_file0 = Dune_file0
 module Package = Package
 module Dialect = Dialect
 module Private_context = Private_context
 module Odoc = Odoc
+module Library = Library
+module Executables = Executables
+module Tests = Tests
+module Stanzas = Stanzas
 
 module Install_rules = struct
   let install_file = Install_rules.install_file
@@ -71,5 +79,14 @@ module For_tests = struct
   module Dynlink_supported = Dynlink_supported
   module Ocamlobjinfo = Ocamlobjinfo
   module Action_unexpanded = Action_unexpanded
-  module Jsoo_rules = Jsoo_rules
+end
+
+module Coq = struct
+  module Coq_mode = Coq_mode
+  module Coq_rules = Coq_rules
+  module Coq_module = Coq_module
+  module Coq_sources = Coq_sources
+  module Coq_lib_name = Coq_lib_name
+  module Coq_lib = Coq_lib
+  module Coq_flags = Coq_flags
 end
