@@ -2,21 +2,21 @@ open Import
 open Memo.O
 module Gen_rules = Build_config.Gen_rules
 
-let install_stanza_rules ~ctx_dir ~expander (install_conf : Dune_file.Install_conf.t) =
+let install_stanza_rules ~ctx_dir ~expander (install_conf : Install_conf.t) =
   let action =
     (* XXX we're evaluating these stanzas here and [Install_rules]. Seems a bit
        sad to do that *)
     let files_and_dirs =
-      let expand_str = Expander.No_deps.expand_str expander in
+      let expand = Expander.No_deps.expand expander ~mode:Single in
       let+ files_expanded =
         Install_entry.File.to_file_bindings_expanded
           install_conf.files
-          ~expand_str
+          ~expand
           ~dir:ctx_dir
       and+ dirs_expanded =
         Install_entry.Dir.to_file_bindings_expanded
           install_conf.dirs
-          ~expand_str
+          ~expand
           ~dir:ctx_dir
           ~relative_dst_path_starts_with_parent_error_when:`Deprecation_warning_from_3_11
       in
@@ -106,26 +106,22 @@ end = struct
      being handled directly in the stanza. *)
   let of_stanza stanza ~sctx ~src_dir ~ctx_dir ~scope ~dir_contents ~expander =
     let dir = ctx_dir in
-    let toplevel_setup = Toplevel.Stanza.setup in
-    let open Dune_file in
-    match stanza with
-    | Toplevel toplevel ->
-      let+ () = toplevel_setup ~sctx ~dir ~toplevel in
+    match Stanza.repr stanza with
+    | Toplevel_stanza.T toplevel ->
+      let+ () = Toplevel.Stanza.setup ~sctx ~dir ~toplevel in
       empty_none
-    | Library lib ->
-      (* XXX why are we setting up private doc rules for disabled libraries? *)
-      let* () = Odoc.setup_private_library_doc_alias sctx ~scope ~dir:ctx_dir lib
-      and+ enabled_if =
-        Lib.DB.available (Scope.libs scope) (Dune_file.Library.best_name lib)
-      in
+    | Library.T lib ->
+      let* enabled_if = Lib.DB.available (Scope.libs scope) (Library.best_name lib) in
       if_available_buildable
         ~loc:lib.buildable.loc
         (fun () -> Lib_rules.rules lib ~sctx ~dir ~scope ~dir_contents ~expander)
         enabled_if
-    | Foreign_library lib ->
-      let+ () = Lib_rules.foreign_rules lib ~sctx ~dir ~dir_contents ~expander in
-      empty_none
-    | Executables exes ->
+    | Foreign.Library.T lib ->
+      Expander.eval_blang expander lib.enabled_if
+      >>= if_available (fun () ->
+        let+ () = Lib_rules.foreign_rules lib ~sctx ~dir ~dir_contents ~expander in
+        empty_none)
+    | Executables.T exes ->
       Expander.eval_blang expander exes.enabled_if
       >>= if_available (fun () ->
         let+ () =
@@ -139,14 +135,14 @@ end = struct
               (List.map exes.names ~f:(fun (_, exe) ->
                  Path.Build.relative dir (exe ^ Js_of_ocaml.Ext.exe)))
         })
-    | Alias alias ->
+    | Alias_conf.T alias ->
       let+ () = Simple_rules.alias sctx alias ~dir ~expander in
       empty_none
-    | Tests tests ->
+    | Tests.T tests ->
       Expander.eval_blang expander tests.build_if
       >>= if_available_buildable ~loc:tests.exes.buildable.loc (fun () ->
         Test_rules.rules tests ~sctx ~dir ~scope ~expander ~dir_contents)
-    | Copy_files { files = glob; _ } ->
+    | Copy_files.T { files = glob; _ } ->
       let+ source_dirs =
         let+ src_glob = Expander.No_deps.expand_str expander glob in
         if Filename.is_relative src_glob
@@ -160,10 +156,10 @@ end = struct
         else None
       in
       { empty_none with source_dirs }
-    | Install i ->
+    | Install_conf.T i ->
       let+ () = install_stanza_rules ~ctx_dir ~expander i in
       empty_none
-    | Plugin p ->
+    | Plugin.T p ->
       let+ () = Plugin_rules.setup_rules ~sctx ~dir p in
       empty_none
     | Cinaps.T cinaps ->
@@ -209,14 +205,9 @@ let define_all_alias ~dir ~project ~js_targets =
   Rules.Produce.Alias.add_deps (Alias.make Alias0.all ~dir) deps
 ;;
 
-let gen_rules_for_stanzas
-  sctx
-  dir_contents
-  cctxs
-  expander
-  { Dune_file.dir = src_dir; stanzas; project }
-  ~dir:ctx_dir
-  =
+let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_dir =
+  let src_dir = Dune_file.dir dune_file in
+  let* stanzas = Dune_file.stanzas dune_file in
   let* { For_stanza.merlin = merlins; cctx = cctxs; js = js_targets; source_dirs } =
     let* scope = Scope.DB.find_by_dir ctx_dir in
     For_stanza.of_stanzas
@@ -238,7 +229,7 @@ let gen_rules_for_stanzas
       ~f:(Merlin.add_rules sctx ~dir:ctx_dir ~more_src_dirs ~expander)
   and+ () =
     Memo.parallel_iter stanzas ~f:(fun stanza ->
-      match (stanza : Stanza.t) with
+      match Stanza.repr stanza with
       | Menhir_stanza.T m ->
         Expander.eval_blang expander m.enabled_if
         >>= (function
@@ -295,26 +286,23 @@ let gen_rules_for_stanzas
         Coq_rules.setup_extraction_rules ~sctx ~dir:ctx_dir ~dir_contents m
       | Coq_stanza.Coqpp.T m -> Coq_rules.setup_coqpp_rules ~sctx ~dir:ctx_dir m
       | _ -> Memo.return ())
-  and+ () = define_all_alias ~dir:ctx_dir ~project ~js_targets in
+  and+ () =
+    let project = Dune_file.project dune_file in
+    define_all_alias ~dir:ctx_dir ~project ~js_targets
+  in
   cctxs
 ;;
 
-let gen_format_and_cram_rules sctx ~expander ~dir source_dir =
-  let+ () = Format_rules.setup_alias sctx ~dir
-  and+ () =
-    Source_tree.Dir.cram_tests source_dir >>= Cram_rules.rules ~sctx ~expander ~dir
-  in
+let gen_format_and_cram_rules sctx ~dir source_dir =
+  let+ () = Format_rules.setup_alias ~dir
+  and+ () = Cram_rules.rules source_dir ~sctx ~dir in
   ()
 ;;
 
 let gen_rules_source_only sctx ~dir source_dir =
   Rules.collect_unit (fun () ->
     let* sctx = sctx in
-    let* expander =
-      let+ expander = Super_context.expander sctx ~dir in
-      Dir_contents.add_sources_to_expander sctx expander
-    in
-    let+ () = gen_format_and_cram_rules sctx ~expander ~dir source_dir
+    let+ () = gen_format_and_cram_rules sctx ~dir source_dir
     and+ () =
       define_all_alias ~dir ~js_targets:[] ~project:(Source_tree.Dir.project source_dir)
     in
@@ -324,14 +312,11 @@ let gen_rules_source_only sctx ~dir source_dir =
 let gen_rules_group_part_or_root sctx dir_contents cctxs ~source_dir ~dir
   : (Loc.t * Compilation_context.t) list Memo.t
   =
-  let* expander =
-    let+ expander = Super_context.expander sctx ~dir in
-    Dir_contents.add_sources_to_expander sctx expander
-  in
-  let* () = gen_format_and_cram_rules sctx ~expander ~dir source_dir
+  let* expander = Super_context.expander sctx ~dir in
+  let* () = gen_format_and_cram_rules sctx ~dir source_dir
   and+ stanzas =
     (* CR-soon rgrinberg: we shouldn't have to fetch the stanzas yet again *)
-    Only_packages.stanzas_in_dir dir
+    Dune_load.stanzas_in_dir dir
     >>= function
     | Some d -> Memo.return (Some d)
     | None ->
@@ -347,6 +332,14 @@ let gen_rules_group_part_or_root sctx dir_contents cctxs ~source_dir ~dir
   contexts
 ;;
 
+(* Warn whenever [(name <name>)]) is missing from the [dune-project] file *)
+let missing_project_name =
+  Warning.make
+    ~default:(fun version -> if version >= (2, 8) then `Enabled else `Disabled)
+    ~name:"missing_project_name"
+    ~since:(3, 11)
+;;
+
 (* To be called once per project, when we are generating the rules for the root
    directory of the project *)
 let gen_project_rules =
@@ -372,21 +365,23 @@ let gen_project_rules =
             version
             ~what:"This field"
     and+ () =
-      match Dune_project.name project with
-      | Named _ -> Memo.return ()
-      | Anonymous _ ->
+      match Dune_project_name.name (Dune_project.name project) with
+      | Some _ -> Memo.return ()
+      | None ->
         (match
-           Dune_project.dune_version project >= (2, 8)
-           && Dune_project.generate_opam_files project
+           if Dune_project.dune_version project >= (2, 8)
+              && Dune_project.generate_opam_files project
+           then Dune_project.file project
+           else None
          with
-         | false -> Memo.return ()
-         | true ->
+         | None -> Memo.return ()
+         | Some project_file ->
            Warning_emit.emit
-             Warning.missing_project_name
+             missing_project_name
              (Warning_emit.Context.project project)
              (fun () ->
                 let+ () = Memo.return () in
-                let loc = Loc.in_file (Path.source (Dune_project.file project)) in
+                let loc = Loc.in_file (Path.source project_file) in
                 User_message.make
                   ~loc
                   [ Pp.text
@@ -681,5 +676,7 @@ let gen_rules ctx ~dir components =
           (Memo.return rules)))
   else if Context_name.equal ctx Private_context.t.name
   then private_context ~dir components ctx
+  else if Context_name.equal ctx Fetch_rules.context.name
+  then Fetch_rules.gen_rules ~dir ~components
   else gen_rules ctx (Super_context.find_exn ctx) ~dir components
 ;;
