@@ -1,9 +1,5 @@
 open Import
 open Memo.O
-module Buildable = Dune_file.Buildable
-module Library = Dune_file.Library
-module Public_lib = Dune_file.Public_lib
-module Mode_conf = Dune_file.Mode_conf
 
 let msvc_hack_cclibs =
   List.map ~f:(fun lib ->
@@ -68,7 +64,7 @@ let build_lib
     in
     let ocaml_flags = Ocaml_flags.get flags (Ocaml mode) in
     let* standard =
-      let+ project = Scope.DB.find_by_dir dir |> Memo.map ~f:Scope.project in
+      let+ project = Dune_load.find_project ~dir in
       match Dune_project.use_standard_c_and_cxx_flags project with
       | Some true when Buildable.has_foreign_cxx lib.buildable ->
         Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
@@ -263,13 +259,10 @@ let foreign_rules (library : Foreign.Library.t) ~sctx ~expander ~dir ~dir_conten
   in
   let* () = Check_rules.add_files sctx ~dir o_files in
   let* standard =
-    let+ project =
-      let+ scope = Scope.DB.find_by_dir dir in
-      Scope.project scope
-    in
-    let ctx = Super_context.context sctx in
+    let+ project = Dune_load.find_project ~dir in
     match Dune_project.use_standard_c_and_cxx_flags project with
     | Some true when Foreign.Sources.has_cxx_sources foreign_sources ->
+      let ctx = Super_context.context sctx in
       Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
     | _ -> Action_builder.return []
   in
@@ -328,14 +321,21 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
            (Compilation_context.ocaml cctx).ocaml_config
     in
     let* standard =
-      let+ project = Scope.DB.find_by_dir dir >>| Scope.project in
+      let+ project = Dune_load.find_project ~dir in
       match Dune_project.use_standard_c_and_cxx_flags project with
       | Some true when Foreign.Sources.has_cxx_sources foreign_sources ->
         Cxx_flags.get_flags ~for_:Link (Context.build_context ctx)
       | _ -> Action_builder.return []
     in
     let c_library_flags =
-      Expander.expand_and_eval_set expander lib.c_library_flags ~standard
+      let open Action_builder.O in
+      let+ c_lib = Expander.expand_and_eval_set expander lib.c_library_flags ~standard
+      and+ ctypes_lib =
+        (* CR rgrinberg: Should we add these flags to :standard? to make
+           it possible for users to remove these *)
+        Ctypes_rules.ctypes_cclib_flags sctx ~expander ~buildable:lib.buildable
+      in
+      c_lib @ ctypes_lib
     in
     let lib_o_files_for_all_modes = Mode.Map.Multi.for_all_modes o_files in
     let for_all_modes = List.rev_append vlib_stubs_o_files lib_o_files_for_all_modes in
@@ -428,13 +428,7 @@ let iter_modes_concurrently (t : _ Ocaml.Mode.Dict.t) ~(f : Ocaml.Mode.t -> unit
   ()
 ;;
 
-let setup_build_archives
-  (lib : Dune_file.Library.t)
-  ~top_sorted_modules
-  ~cctx
-  ~expander
-  ~lib_info
-  =
+let setup_build_archives (lib : Library.t) ~top_sorted_modules ~cctx ~expander ~lib_info =
   let obj_dir = Compilation_context.obj_dir cctx in
   let dir = Compilation_context.dir cctx in
   let flags = Compilation_context.flags cctx in
@@ -530,9 +524,9 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
   let requires_link = Lib.Compile.requires_link compile_info in
   let modes =
     let { Lib_config.has_native; _ } = ocaml.lib_config in
-    Dune_file.Mode_conf.Lib.Set.eval_detailed lib.modes ~has_native
+    Mode_conf.Lib.Set.eval_detailed lib.modes ~has_native
   in
-  let package = Dune_file.Library.package lib in
+  let package = Library.package lib in
   let js_of_ocaml = Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml in
   (* XXX(anmonteiro): `public_lib_name` is used to derive Melange's
      `--bs-package-name` argument. We only use the library name for public
@@ -569,6 +563,7 @@ let library_rules
   ~source_modules
   ~dir_contents
   ~compile_info
+  ~ctx_dir
   =
   let source_modules =
     Modules.fold_user_written source_modules ~init:[] ~f:(fun m acc -> m :: acc)
@@ -592,13 +587,19 @@ let library_rules
   let* () =
     Memo.Option.iter vimpl ~f:(Virtual_rules.setup_copy_rules_for_impl ~sctx ~dir)
   in
+  let* expander = Super_context.expander sctx ~dir in
   let* () = Check_rules.add_cycle_check sctx ~dir top_sorted_modules in
   let* () = gen_wrapped_compat_modules lib cctx
   and* () = Module_compilation.build_all cctx
-  and* expander = Super_context.expander sctx ~dir
   and* lib_info =
     let lib_config = ocaml.lib_config in
-    let* info = Library.to_lib_info lib ~dir ~lib_config in
+    let info =
+      Library.to_lib_info
+        lib
+        ~expander:(Memo.return (Expander.to_expander0 expander))
+        ~dir
+        ~lib_config
+    in
     let mode = Lib_mode.Map.Set.for_merlin (Lib_info.modes info) in
     let+ () = Check_rules.add_obj_dir sctx ~obj_dir mode in
     info
@@ -620,6 +621,7 @@ let library_rules
           ~requires:requires_compile
           ~dir_contents
           ~vlib_stubs_o_files)
+  and+ () = Odoc.setup_private_library_doc_alias sctx ~scope ~dir:ctx_dir lib
   and+ () = Odoc.setup_library_odoc_rules cctx local_lib
   and+ () =
     Sub_system.gen_rules
@@ -661,7 +663,14 @@ let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope =
       | Some _ ->
         Ctypes_rules.gen_rules ~loc:(fst lib.name) ~cctx ~buildable ~sctx ~scope ~dir
     in
-    library_rules lib ~local_lib ~cctx ~source_modules ~dir_contents ~compile_info
+    library_rules
+      lib
+      ~local_lib
+      ~cctx
+      ~source_modules
+      ~dir_contents
+      ~compile_info
+      ~ctx_dir:dir
   in
   let* () = Buildable_rules.gen_select_rules sctx compile_info ~dir in
   Buildable_rules.with_lib_deps (Super_context.context sctx) compile_info ~dir ~f
