@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 let remove_extension file =
   let dir = Path.Build.parent_exn file in
@@ -175,9 +176,7 @@ module Processed = struct
       | opens ->
         make_directive
           "FLG"
-          (Sexp.List
-             (List.concat_map opens ~f:(fun name ->
-                [ Sexp.Atom "-open"; Atom (Module_name.to_string name) ])))
+          (Sexp.List (Ocaml_flags.open_flags opens |> List.map ~f:(fun x -> Sexp.Atom x)))
         :: flags
     in
     let suffixes =
@@ -253,17 +252,18 @@ module Processed = struct
     match load_file path with
     | Error msg -> Printf.eprintf "%s\n" msg
     | Ok { per_module_config; pp_config; config } ->
-      let pp_one { module_; opens } =
+      let pp_one (source, { module_; opens }) =
         let open Pp.O in
         let name = Module.name module_ in
         let pp = Module_name.Per_item.get pp_config name in
         let sexp = to_sexp ~opens ~pp config in
-        Pp.vbox (Pp.text (Module_name.to_string name))
+        Pp.hvbox
+          (Pp.textf "%s: %s" (Module_name.to_string name) (Path.Build.to_string source))
         ++ Pp.newline
         ++ Pp.vbox (Sexp.pp sexp)
       in
       let pp =
-        Path.Build.Map.values per_module_config
+        Path.Build.Map.to_list per_module_config
         |> Pp.concat_map ~sep:Pp.cut ~f:pp_one
         |> Pp.vbox
       in
@@ -415,7 +415,7 @@ module Unprocessed = struct
        | Some args ->
          let action =
            let action = Action_unexpanded.Run args in
-           let chdir = Expander.context expander |> Context.build_dir in
+           let chdir = Expander.context expander |> Context_name.build_dir in
            Action_unexpanded.expand_no_targets
              ~loc
              ~expander
@@ -455,7 +455,9 @@ module Unprocessed = struct
     | Pps { loc; pps; flags; staged = _ } ->
       let open Action_builder.O in
       let+ exe, flags =
-        let scope = Expander.scope expander in
+        let* scope =
+          Expander.dir expander |> Scope.DB.find_by_dir |> Action_builder.of_memo
+        in
         Preprocessing.get_ppx_driver ctx ~loc ~expander ~lib_name ~flags ~scope pps
       in
       let args = encode_command ~bin:(Path.build exe) ~args:("--as-ppx" :: flags) in
@@ -463,17 +465,13 @@ module Unprocessed = struct
   ;;
 
   let src_dirs sctx lib =
-    let info = Lib.info lib in
-    match
-      let obj_dir = Lib_info.obj_dir info in
-      Path.is_managed (Obj_dir.byte_dir obj_dir)
-    with
-    | false -> Memo.return (Path.Set.singleton (Lib_info.src_dir info))
+    match Lib.is_local lib with
+    | false -> Lib.info lib |> Lib_info.src_dir |> Path.Set.singleton |> Memo.return
     | true ->
-      let open Memo.O in
-      let+ modules = Dir_contents.modules_of_lib sctx lib in
-      let modules = Option.value_exn modules in
-      Path.Set.map ~f:Path.drop_optional_build_context (Modules.source_dirs modules)
+      Dir_contents.modules_of_lib sctx lib
+      >>| Option.value_exn
+      >>| Modules.source_dirs
+      >>| Path.Set.map ~f:Path.drop_optional_build_context
   ;;
 
   module Per_item_action_builder =
@@ -525,7 +523,7 @@ module Unprocessed = struct
         | Melange ->
           Action_builder.of_memo
             (let open Memo.O in
-             let scope = Expander.scope expander in
+             let* scope = Scope.DB.find_by_dir (Expander.dir expander) in
              let libs = Scope.libs scope in
              Lib.DB.find libs (Lib_name.of_string "melange")
              >>= function
@@ -574,7 +572,7 @@ module Unprocessed = struct
         |> Path.Build.Set.fold ~init ~f:(fun src acc ->
           let config =
             { Processed.module_ = Module.set_pp m None
-            ; opens = Modules.alias_for modules m |> List.map ~f:Module.name
+            ; opens = Modules.local_open modules m
             }
           in
           (src, config) :: acc))
