@@ -104,44 +104,32 @@ let default_build_command =
        then from_2_9
        else
          from_3_0
-           ~with_subst:(Subst_config.is_enabled (Dune_project.subst_config project))
+           ~with_subst:(Toggle.enabled (snd (Dune_project.subst_config project)))
            ~with_sites:Dune_project.(is_extension_set project dune_site_extension))
 ;;
 
-let package_fields
-  { Package.synopsis
-  ; description
-  ; depends
-  ; conflicts
-  ; depopts
-  ; info = _
-  ; id = _
-  ; version = _
-  ; has_opam_file = _
-  ; tags
-  ; loc = _
-  ; deprecated_package_names = _
-  ; sites = _
-  ; opam_file = _
-  ; allow_empty = _
-  }
-  ~project
-  =
+let package_fields package ~project =
   let open Opam_file.Create in
-  let tags = if tags = [] then [] else [ "tags", string_list tags ] in
+  let tags =
+    let tags = Package.tags package in
+    if tags = [] then [] else [ "tags", string_list tags ]
+  in
   let optional =
-    [ "synopsis", synopsis; "description", description ]
+    [ "synopsis", Package.synopsis package; "description", Package.description package ]
     |> List.filter_map ~f:(fun (k, v) ->
       match v with
       | None -> None
       | Some v -> Some (k, string v))
   in
   let dep_fields =
-    [ "depends", depends; "conflicts", conflicts; "depopts", depopts ]
+    [ "depends", Package.depends package
+    ; "conflicts", Package.conflicts package
+    ; "depopts", Package.depopts package
+    ]
     |> List.filter_map ~f:(fun (k, v) ->
       match v with
       | [] -> None
-      | _ :: _ -> Some (k, list Package.Dependency.opam_depend v))
+      | _ :: _ -> Some (k, list Dune_pkg.Package_dependency.opam_depend v))
   in
   let fields = [ optional; dep_fields ] in
   let fields =
@@ -155,17 +143,17 @@ let dune_name = Package.Name.of_string "dune"
 let odoc_name = Package.Name.of_string "odoc"
 
 let insert_dune_dep depends dune_version =
-  let constraint_ : Package.Dependency.Constraint.t =
+  let constraint_ : Package_constraint.t =
     let dune_version = Dune_lang.Syntax.Version.to_string dune_version in
     Uop (Gte, String_literal dune_version)
   in
   let rec loop acc = function
     | [] ->
       let dune_dep =
-        { Package.Dependency.name = dune_name; constraint_ = Some constraint_ }
+        { Package_dependency.name = dune_name; constraint_ = Some constraint_ }
       in
       dune_dep :: List.rev acc
-    | (dep : Package.Dependency.t) :: rest ->
+    | (dep : Package_dependency.t) :: rest ->
       if Package.Name.equal dep.name dune_name
       then (
         let dep =
@@ -186,19 +174,19 @@ let insert_dune_dep depends dune_version =
   loop [] depends
 ;;
 
-let rec already_requires_odoc : Package.Dependency.Constraint.t -> bool = function
-  | Bvar { name = "with-doc" | "build" | "post" } | Uop _ | Bop _ -> true
-  | Bvar _ -> false
+let rec already_requires_odoc : Package_constraint.t -> bool = function
+  | Uop _ | Bop _ -> true
+  | Bvar var -> Dune_lang.Package_variable_name.(one_of var [ with_doc; build; post ])
   | And l -> List.for_all ~f:already_requires_odoc l
   | Or l -> List.exists ~f:already_requires_odoc l
 ;;
 
 let insert_odoc_dep depends =
-  let with_doc : Package.Dependency.Constraint.t = Bvar { name = "with-doc" } in
-  let odoc_dep = { Package.Dependency.name = odoc_name; constraint_ = Some with_doc } in
+  let with_doc : Package_constraint.t = Bvar Dune_lang.Package_variable_name.with_doc in
+  let odoc_dep = { Package_dependency.name = odoc_name; constraint_ = Some with_doc } in
   let rec loop acc = function
     | [] -> List.rev (odoc_dep :: acc)
-    | (dep : Package.Dependency.t) :: rest ->
+    | (dep : Package_dependency.t) :: rest ->
       if Package.Name.equal dep.name odoc_name
          && Option.forall ~f:already_requires_odoc dep.constraint_
       then (* Stop now as odoc will be required anyway *)
@@ -214,34 +202,35 @@ let opam_fields project (package : Package.t) =
   let package =
     if dune_version < (1, 11) || Package.Name.equal package_name dune_name
     then package
-    else { package with depends = insert_dune_dep package.depends dune_version }
+    else
+      Package.map_depends package ~f:(fun depends -> insert_dune_dep depends dune_version)
   in
   let package =
     if dune_version < (2, 7) || Package.Name.equal package_name odoc_name
     then package
-    else { package with depends = insert_odoc_dep package.depends }
+    else Package.map_depends package ~f:insert_odoc_dep
   in
   let package_fields = package_fields package ~project in
   let open Opam_file.Create in
-  let info = package.Package.info in
+  let info = Package.info package in
   let optional_fields =
-    [ "bug-reports", Package.Info.bug_reports info
-    ; "homepage", Package.Info.homepage info
-    ; "doc", Package.Info.documentation info
+    [ "bug-reports", Package_info.bug_reports info
+    ; "homepage", Package_info.homepage info
+    ; "doc", Package_info.documentation info
     ; ( "license"
-      , match Package.Info.license info with
+      , match Package_info.license info with
         | Some [ x ] -> Some x
         | _ -> None )
-    ; "version", Option.map ~f:Package_version.to_string package.version
-    ; "dev-repo", Option.map ~f:Package.Source_kind.to_string (Package.Info.source info)
+    ; "version", Option.map ~f:Package_version.to_string (Package.version package)
+    ; "dev-repo", Option.map ~f:Source_kind.to_string (Package_info.source info)
     ]
     |> List.filter_map ~f:(fun (k, v) -> Option.map v ~f:(fun v -> k, string v))
   in
   let list_fields =
-    [ "maintainer", Package.Info.maintainers info
-    ; "authors", Package.Info.authors info
+    [ "maintainer", Package_info.maintainers info
+    ; "authors", Package_info.authors info
     ; ( "license"
-      , match Package.Info.license info with
+      , match Package_info.license info with
         | None | Some [ _ ] -> None
         | Some l -> Some l )
     ]
@@ -328,14 +317,14 @@ let add_opam_file_rule sctx ~project ~pkg =
 let add_opam_file_rules sctx project =
   Memo.when_ (Dune_project.generate_opam_files project) (fun () ->
     let packages = Dune_project.packages project in
-    Package.Name.Map_traversals.parallel_iter packages ~f:(fun _name (pkg : Package.t) ->
+    Package_map_traversals.parallel_iter packages ~f:(fun _name (pkg : Package.t) ->
       add_opam_file_rule sctx ~project ~pkg))
 ;;
 
 let add_rules sctx project =
   Memo.when_ (Dune_project.generate_opam_files project) (fun () ->
     let packages = Dune_project.packages project in
-    Package.Name.Map_traversals.parallel_iter packages ~f:(fun _name (pkg : Package.t) ->
+    Package_map_traversals.parallel_iter packages ~f:(fun _name (pkg : Package.t) ->
       let* () =
         add_alias_rule (Context.build_context (Super_context.context sctx)) ~project ~pkg
       in
