@@ -2,6 +2,33 @@
 
 open Import
 
+module Lock_dir : sig
+  type t =
+    { path : Path.Source.t
+    ; version_preference : Dune_pkg.Version_preference.t option
+    ; solver_env : Dune_pkg.Solver_env.t option
+    ; unset_solver_vars : Dune_lang.Package_variable_name.Set.t option
+    ; repositories : (Loc.t * Dune_pkg.Pkg_workspace.Repository.Name.t) list
+    ; constraints : Dune_lang.Package_dependency.t list
+    ; pins : (Loc.t * string) list
+    }
+
+  val equal : t -> t -> bool
+  val to_dyn : t -> Dyn.t
+end
+
+module Lock_dir_selection : sig
+  (** A dsl for selecting a lockdir either by literally naming it or using a
+      cond expression to select a lockdir based on blangs *)
+  type t
+
+  val eval
+    :  t
+    -> dir:Path.Source.t
+    -> f:Value.t list Memo.t String_with_vars.expander
+    -> Path.Source.t Memo.t
+end
+
 module Context : sig
   module Target : sig
     type t =
@@ -16,7 +43,7 @@ module Context : sig
       { loc : Loc.t
       ; profile : Profile.t
       ; targets : Target.t list
-      ; env : Dune_env.Stanza.t option
+      ; env : Dune_env.t option
       ; toolchain : Context_name.t option
       ; name : Context_name.t
       ; host_context : Context_name.t option
@@ -46,10 +73,7 @@ module Context : sig
   module Default : sig
     type t =
       { base : Common.t
-      ; lock : Path.Source.t option
-      ; version_preference : Dune_pkg.Version_preference.t option
-      ; solver_sys_vars : Dune_pkg.Solver_env.Variable.Sys.Bindings.t option
-      ; repositories : Dune_pkg.Pkg_workspace.Repository.Name.t list
+      ; lock_dir : Lock_dir_selection.t option
       }
   end
 
@@ -59,7 +83,7 @@ module Context : sig
 
   val loc : t -> Loc.t
   val name : t -> Context_name.t
-  val env : t -> Dune_env.Stanza.t option
+  val env : t -> Dune_env.t option
   val host_context : t -> Context_name.t option
   val to_dyn : t -> Dyn.t
   val base : t -> Common.t
@@ -78,14 +102,19 @@ end
 type t = private
   { merlin_context : Context_name.t option
   ; contexts : Context.t list
-  ; env : Dune_env.Stanza.t option
+  ; env : Dune_env.t option
   ; config : Dune_config.t
   ; repos : Dune_pkg.Pkg_workspace.Repository.t list
+  ; lock_dirs : Lock_dir.t list
+  ; dir : Path.Source.t
+  ; sources : Dune_pkg.Pin_stanza.DB.Workspace.t
   }
 
 val equal : t -> t -> bool
 val to_dyn : t -> Dyn.t
 val hash : t -> int
+val find_lock_dir : t -> Path.Source.t -> Lock_dir.t option
+val default_repositories : Dune_pkg.Pkg_workspace.Repository.t list
 
 module Clflags : sig
   type t =
